// Sample01.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
static
void ListPorts();

static
void PrintBapPortDetails(TC_PORT port);

static
void PrintT2apPortDetails(TC_PORT port);

int main(int argc, char* argv[])
{
	UNUSED(argc);
	UNUSED(argv);

	ListPorts();

	return 0;
}

void ListPorts()
{
	PTC_PORT	portList;
	ULONG		numPorts;
	TC_STATUS		status;
	ULONG		i;

	status = TcQueryPortList(&portList, &numPorts);

	if (status == TC_SUCCESS)
	{
		for (i = 0; i < numPorts; i++)
		{
			PCHAR description;
			PCHAR name;
			TC_PORT_TYPE portType;
			TC_BOARD board;
			TC_BOARD_TYPE boardType = TC_BOARD_TYPE_UNKNOWN;

			TC_PORT port = portList[i];
			
			name = TcPortGetName(port);
			description = TcPortGetDescription(port);
			portType = TcPortGetType(port);
			

			printf("%02u. Name: %s\n", i, name);
			printf("    Description: %s\n", description);
			printf("    Port type: ");
			switch(portType)
			{
			case TC_PORT_TYPE_PHYSICAL: 
				printf("Physical\n");
				if (TcPortQueryBoard(port, &board) != TC_SUCCESS)
				{
					board = NULL;
				}
				else
				{
					boardType = TcBoardGetType(board);
				}
				printf("BoardType: ");
				if (board != NULL)
				{
					switch(boardType)
					{
					case TC_BOARD_TYPE_9402PT: printf("9402PT"); break;
					default: printf("<unknown>");
					}
				}
				else
				{
					printf("NO BOARD");
				}

				break;

			case TC_PORT_TYPE_BAP: 
				printf("BAP\n"); 
				PrintBapPortDetails(port);
				break;

			case TC_PORT_TYPE_TCAP: 
				printf("TcAP\n"); 
				PrintT2apPortDetails(port);
				break;

			default:
				printf("Unknown");
			}
			printf("\n");
		}

		status = TcFreePortList(portList);
	
	}
	else
	{
		printf("Error getting port list %s [%08x]\n", TcStatusGetString(status), status);
	}
}

void PrintBapPortDetails(TC_PORT port)
{
	PTC_PORT aggregatedPorts;
	TC_BOARD board;
	ULONG numPorts;
	TC_STATUS status;
	ULONG i;
	PCHAR description;
	PCHAR name;
	PCHAR boardDescription;

	//
	// Print the board information
	//
	status = TcPortQueryBoard(port, &board);

	if (status == TC_SUCCESS)
	{
		boardDescription = TcBoardGetDescription(board);
		printf("  Board description: %s\n", boardDescription);
	}

	//
	// list the ports aggregated
	//
	status = TcPortQueryAggregatedPortList(port, &aggregatedPorts, &numPorts);

	if (status == TC_SUCCESS)
	{
		printf("  Aggregated:\n");

		for (i = 0; i < numPorts; i++)
		{
			name = TcPortGetName(aggregatedPorts[i]);
			description = TcPortGetDescription(aggregatedPorts[i]);
			printf("    %u. %s (%s)\n", i, description, name);
		}
	}
}

void PrintT2apPortDetails(TC_PORT port)
{
	PTC_PORT aggregatedPorts;
	ULONG numPorts;
	TC_STATUS status;
	ULONG i;
	PCHAR description;
	PCHAR name;

	//
	// list the ports aggregated
	//
	status = TcPortQueryAggregatedPortList(port, &aggregatedPorts, &numPorts);

	if (status == TC_SUCCESS)
	{
		printf("  Aggregated:\n");

		for (i = 0; i < numPorts; i++)
		{
			name = TcPortGetName(aggregatedPorts[i]);
			description = TcPortGetDescription(aggregatedPorts[i]);
			printf("    %u. %s (%s)\n", i, description, name);
		}
	}
}
